#!/usr/bin/env python
import sys
import os
import subprocess
import logging
import argparse

sys.path.append('/sw/castor/Script')
import cs.Tools.Libraries.Database as Oracle
import cs.Tools.FilenameParsers.SFOFileNameParser as SFOFileNameParser


# Config for cs...Database: only needs a few paramaters of the CS config
class MockConfig:
    def __init__(self, url):
        self.connection = url
        self.file_table = 'SFO_TZ_FILE'
        self.lb_table = 'SFO_TZ_LUMIBLOCK'
        self.run_table = 'SFO_TZ_RUN'

def oracle_check_closed_ondisk(oradb_url, run_number, outliers_file,
        verbose=False, dryrun=False, report=None):
    conf = MockConfig(oradb_url)
    parser = SFOFileNameParser.SFOFileNameParser
    dblogger = logging.getLogger()
    dblogger.setLevel(logging.ERROR)
    dblogger.addHandler(logging.StreamHandler(sys.stdout))
    oradb = Oracle.Database(conf, dblogger, parser)

    updated = 0
    if report: reportf = open(report, 'a')

    ora_state_3 = oradb.db.curs.execute("""select SFOID, SFOPFN from SFO_TZ_FILE
            where RUNNR=:runnr and FILESTATE='CLOSED' and TRANSFERSTATE='ONDISK'
            and SFOID like 'SFO%'""", runnr=run_number).fetchall()
    if verbose: print(f'Found {len(ora_state_3)} (closed,ondisk)')

    with open(outliers_file, 'w') as outf:
        for [sfoid, sfopfn] in ora_state_3:
            cmd = ['ssh', '-x', f'pc-tdq-{sfoid.lower()}', 'ls', f'{sfopfn}.COPIED']
            lsp = subprocess.run(cmd, check=True, stdout=subprocess.PIPE,
                    stderr=subprocess.STDOUT)
            if lsp.returncode == 0:
                # .COPIED file exists: mark file as transferred in DB

                # Get remote dir from .COPIED file
                cmd = ['ssh', '-x', f'pc-tdq-{sfoid.lower()}', 'grep', 'RemoteDir',
                        f'{sfopfn}.COPIED']
                try:
                    output = subprocess.check_output(cmd)
                    remote_dir = output.decode().split()[2]
                except subprocess.CalledProcessError:
                    outf.write(f'(closed,ondisk), error reading RemoteDir from'
                            ' COPIED file: {sfoid.lower()} {sfopfn}\n')
                    continue

                remote_filename = os.path.join(remote_dir, os.path.basename(sfopfn))
                if report: reportf.write(f'marking {sfopfn} as transferred in DB\n')
                if not dryrun:
                    oradb.Transfer(sfopfn, remote_filename)
                updated += 1
            else:
                # No .COPIED file: why couldn't CS transfer it?
                outf.write(f'(closed,ondisk), no COPIED file: {sfoid.lower()} {sfopfn}\n')

    oradb.db.orcl.close()
    if report: reportf.close()
    if verbose:
        if dryrun:
            print(f'would have marked {updated} as transferred')
        else:
            print(f'marked {updated} as transferred')


if __name__ == '__main__':
    ap = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    ap.add_argument("oradb_url",
            help="URL of the oracle DB to fix, example: oracle://int8r/ATLAS_SFO_T0")
    ap.add_argument("run_number", type=int,
            help="number of the run to check")
    ap.add_argument("outliers_file",
            help="file to print out outliers")
    ap.add_argument("-v", "--verbose", action="store_true",
            help="enable verbosity")
    ap.add_argument("-d", "--dryrun", action="store_true",
            help="do not modify oracle DB")
    ap.add_argument("-r", "--report",
            help="file-by-file report")
    args = ap.parse_args()

    oracle_check_closed_ondisk(args.oradb_url, args.run_number, args.outliers_file,
            verbose=args.verbose, dryrun=args.dryrun, report=args.report)
