import socket
import os
import logging
import mailinglogger

def make_is_data_identifier(is_server,tdaq_app_name):
    return "{}.{}.State".format(is_server,tdaq_app_name)

def make_fs_data_identifier(is_server,tdaq_app_name,fs_name):
    return "{}.{}.fsinfo-{}".format(is_server,tdaq_app_name,fs_name)

def make_tdaq_app_name(config):
    hostname = socket.gethostname()
    cleanFileName = config.FileName.replace("Config", "",1).replace("Conf","",1).replace(".cfg","",1).replace("-","").replace("_","")
    if len(cleanFileName) == 0:
        return "CastorScript-{}".format(hostname)
    return "CastorScript-{}-{}".format(hostname, cleanFileName)

def enable_root_logger():
    #let all logs get to the root logger
    #(log levels are set in each handler)
    logging.getLogger().setLevel("DEBUG")

def enable_mail_logging(config):
    # by default max email sending rate is 10 per hour
    mail_handler = mailinglogger.MailingLogger(config.EmailLogSender,
            config.EmailLogList, subject='CastorScript message',
            mailhost='localhost')

    mail_handler.setFormatter(formatter)
    set_log_level(config.EmailLogLevel, mail_handler)
    logging.getLogger().addHandler(mail_handler)


#### Set Log severity level #####
def set_log_level(level, logger):

    level = level.upper()

    levelmap = {'DEBUG':logging.DEBUG,
                'INFO':logging.INFO,
                'WARNING':logging.WARNING,
                'ERROR':logging.ERROR,
                'CRITICAL':logging.CRITICAL}

    logger.setLevel(levelmap[level])

formatter = logging.Formatter('%(asctime)s %(levelname)-8s [%(filename)s:%(lineno)d] %(message)s')

def enable_file_logging(loggerName, fileName, filePath, logLevel):
    """Enables file logging to a file with fileName at filePath.
    It will make a logger with loggerName as identifier, at the specified loglevel,

    It will afterwards be retrievable with the use of logging.getLogger(loggerName), from any thread as long as it is in the same process.

    Filename should formatted like this: name + Log.out

    returns a reference to a properly configured logger"""

    ##### Setup FileHandler for Logger #####
    logFilePath = os.path.join(str(filePath),fileName)
    fileHandler = logging.FileHandler(logFilePath,'w')
    fileHandler.setFormatter(formatter)

    ##### Set LogLevel for filehandler #####
    set_log_level(logLevel, fileHandler)
    logger = logging.getLogger(loggerName)
    logger.addHandler(fileHandler)

    return logger
